package imm.se.acceptance_tests;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;

import cucumber.api.java.en.Given;
import cucumber.api.java.en.Then;
import cucumber.api.java.en.When;
import imm.se.vending.model.Fruit;
import imm.se.vending.model.VendingMachine;

public class VendingMachineSteps {
	
	private VendingMachine vendingMachine;
	
	private int restMoney = 0;
	private int totalEarnings = 0;
	private int inputMoney = 0;


	@Given("^the vending machine has (\\d+) bananas and (\\d+) apples$")
	public void theVendingMachineHasBananasAndApples(int numberOfBananas, int numberOfApples) throws Exception {
		vendingMachine = new VendingMachine(numberOfBananas, numberOfApples);
	}

	@Given("^a banana costs (\\d+) kr and an apple (\\d+) kr$")
	public void aBananaCostsKrAndAnAppleKr(int banana, int apple) throws Exception {
		assertEquals(apple, vendingMachine.getPrice(Fruit.APPLE));
		assertEquals(banana, vendingMachine.getPrice(Fruit.BANANA));
	}

	@When("^user Bob enters (\\d+) kr$")
	public void userBobEntersKr(int money) throws Exception {
		vendingMachine.input(money);
	}

	@When("^Bob presses the button for \"([^\"]*)\"$")
	public void bobPressesTheButtonFor(String fruit) throws Exception {
		vendingMachine.selectFruit(Fruit.valueOf(fruit));
	}

	@When("^Bob presses the cancel button$")
	public void bobPressesTheCancelButton() throws Exception {
		vendingMachine.cancel();
	}

	@Then("^an \"([^\"]*)\" is dispensed$")
	public void anIsDispensed(String fruit) throws Exception {
		assertEquals(Fruit.valueOf(fruit), vendingMachine.getDispensedItem());
	}
	
	@Then("^a \"([^\"]*)\" is dispensed$")
	public void aIsDispensed(String fruit) throws Exception {
		assertEquals(Fruit.valueOf(fruit), vendingMachine.getDispensedItem());
	}


	@Then("^nothing is dispensed$")
	public void nothingIsDispensed() throws Exception {
		assertNull(vendingMachine.getDispensedItem());
	}

	@Then("^the machine has earned (\\d+) kr$")
	public void theMachineHasEarnedKr(int earnedMoney) throws Exception {
		assertEquals(earnedMoney, vendingMachine.getTotalMoney());
	}

	@Then("^no money is returned$")
	public void noMoneyIsReturned() throws Exception {
		assertEquals(0, vendingMachine.getRest());
	}

	@Then("^(\\d+) kr is returned$")
	public void krIsReturned(int restMoney) throws Exception {
		assertEquals(restMoney, vendingMachine.getRest());
	}

	@Then("^(\\d+) kr are returned$")
	public void krAreReturned(int restMoney) throws Exception {
		assertEquals(restMoney, vendingMachine.getRest());
	}
	
	@Given("^the vending machine has fruits$")
	public void theVendingMachineHasFruits() throws Exception {
	    vendingMachine = new VendingMachine(2,2);
	}
	
	@When("^the user enters enough money for (?:a|the) fruit$")
	public void theUserEntersEnoughMoneyForAFruit() throws Exception {
		inputMoney = 5;
		vendingMachine.input(inputMoney);
		restMoney = 2;
		totalEarnings = 3;
	}
	
	@When("^(?:the|a) user selects (?:the|a) fruit$")
	public void theUserSelectsTheFruit() throws Exception {
		vendingMachine.selectFruit(Fruit.APPLE);
	}
	
	@Then("^the fruit will be dispensed$")
	public void theFruitWillBeDispensed() throws Exception {
		assertEquals(Fruit.APPLE, vendingMachine.getDispensedItem());
	}
	
	@Then("^the machine returns the rest money$")
	public void theMachineReturnsTheRestMoney() throws Exception {
	    assertEquals(restMoney,vendingMachine.getRest());
	}
	
	@Then("^the machine remembers its earnings$")
	public void theMachineRemembersItsEarnings() throws Exception {
	    assertEquals(totalEarnings, vendingMachine.getTotalMoney());
	}
	
	@Given("^the vending machine has no fruits$")
	public void theVendingMachineHasNoFruits() throws Exception {
	    vendingMachine = new VendingMachine(0,0);
	}
	
	@Then("^the machine has earned nothing$")
	public void theMachineHasEarnedNothing() throws Exception {
	    assertEquals(0,vendingMachine.getTotalMoney());
	}
	
	@When("^the user cancels the operation$")
	public void theUserCancelsTheOperation() throws Exception {
	    vendingMachine.cancel();
	}
	
	@Then("^the machine returns the money$")
	public void theMachineReturnsTheMoney() throws Exception {
	    assertEquals(inputMoney, vendingMachine.getRest());
	}
	
	@Given("^the user enters not enough money for the first fruit$")
	public void theUserEntersNotEnoughMoneyForTheFirstFruit() throws Exception {
	    inputMoney = 3;
	    vendingMachine.input(inputMoney);
	}

	@Given("^the user selects the first fruit$")
	public void theUserSelectsTheFirstFruit() throws Exception {
		vendingMachine.selectFruit(Fruit.BANANA);
	}

	@When("^the user selects a second fruit for which he entered enough money$")
	public void theUserSelectsASecondFruitForWhichHeEnteredEnoughMoney() throws Exception {
		vendingMachine.selectFruit(Fruit.APPLE);
		restMoney = 0;
		totalEarnings = 3;
	}

}
